/* El juego de las Torres de Hanoi fue inventado a fines del siglo XIX y es un problema interesante para resolver de forma recursiva. Consiste en tres pilas (o varillas) verticales denominadas origen, destino y auxiliar. En la pila de origen hay un nmero n de discos, dispuestos de mayor a menor tamao, como se muestra en la figura siguiente: El objetivo del rompecabezas es mover todos los discos desde la pila de origen hacia la de destino, haciendo uso de la varilla auxiliar. Aunque su solucin parezca trivial, deben respetarse las reglas que siguen:
/* Slo podr desplazarse el disco que se encuentre en el tope de la pila.*/
/* En ningn momento podr haber un disco de mayor tamao sobre otro ms pequeo.*/
/* Los discos estn numerados desde 1 hasta n (el disco 1 es el ms pequeo, y el n, el ms grande).*/
/* Se recomienda que, luego de analizar este caso, el lector implemente la solucin a este problema de manera iterativa (no recursiva) mediante estructuras de control repetitivas.*/
/* Solucin en lenguaje C */

#include <stdio.h>
#define ORIGEN 1
#define DESTINO 2
#define AUXILIAR 3

void hanoi(int cant_discos, int origen, int destino, int auxiliar)
{

	/* No hay ms discos para mover */
	if (cant_discos == 0)
	return;

	/* Se mueven todos los discos, salvo el de ms abajo, a la pila auxiliar */
	hanoi(cant_discos - 1, origen, auxiliar, destino);

	/* Una vez que ha quedado libre, se mueve el ltimo disco */
	/* a la pila de destino */
	printf("Moviendo disco %d desde la pila %d hacia la pila %d\n", cant_discos, origen, destino);

	/* Por ltimo se mueven todos los discos que estaban (temporalmente) en */
	/* la pila auxiliar hacia la pila de destino. */
	hanoi(cant_discos - 1, auxiliar, destino, origen);
}
int main()
{
	int cant_discos;
	printf("Juego de las Torres de Hanoi\n");
	printf("****************************\n\n");

	printf("Ingrese la cantidad de discos: ");
	scanf("%d", &cant_discos);
	if (cant_discos > 0)
	{
		hanoi(cant_discos, ORIGEN, DESTINO, AUXILIAR);
		printf("FIN DEL ROMPECABEZAS!\n\n");
	}
	else
	{
	printf("La cantidad de discos a utilizar debe ser mayor que cero!\n");
	}
	return 0;
}
